import {
  CheckDatatypeConstraint,
  GetAttributeBoolOperators,
} from '@graphpolaris/querybuilder/usecases';
import { useTheme } from '@mui/material';
import React, { useMemo, useState } from 'react';
import { Handle, Position } from 'react-flow-renderer';
import { Handles } from '../entitypill/entitypill';
import styles from './attributepill.module.scss';
import AttributeOperatorSelect from './operatorselect';

/**
 * Component to render an attribute flow element
 * @param {FlowElement<EntityData>)} param0 The data of an entity flow element.
 */
export const AttributeRFPill = React.memo(({ data }: { data: any }) => {
  const theme = useTheme();
  const [readonly, setReadonly] = useState(true);
  const [value, setValue] = useState(data?.value || '');

  const onChange = (e: any) => {
    setValue(e.target.value);
  };
  const validateInput = () => {
    setValue(CheckDatatypeConstraint(data.datatype, value));
  };

  // Calculates the size of the input
  const getInputWidth = () => {
    if (value == '') return 1;
    else if (value.length > 10) return 10;
    return value.length;
  };

  const boolOperators = useMemo(
    () => GetAttributeBoolOperators(data?.datatype),
    [data?.datatype]
  );

  return (
    <div
      className={styles.attribute}
      style={{
        background: theme.palette.queryBuilder.attribute.background,
        color: theme.palette.queryBuilder.text,
      }}
    >
      {/* <Handle
        id={Handles.Attribute}
        type="source"
        position={Position.Bottom}
        className={styles.handle}
      /> */}
      <div className={styles.contentWrapper}>
        <span className={styles.content} title={data.name}>
          {data.name}
        </span>
        <AttributeOperatorSelect
          selected={data?.operator}
          options={boolOperators}
          changed={(o) => console.log(o)}
        />
        <span className={styles.attributeInput}>
          <input
            style={{ maxWidth: `${getInputWidth()}ch` }}
            type="string"
            placeholder={'?'}
            value={value}
            onChange={onChange}
            onBlur={validateInput}
            onKeyDown={(e) => e.key == 'Enter' && validateInput()}
          ></input>
        </span>
      </div>
    </div>
  );
});

export default AttributeRFPill;
