import React from 'react';
import {
  colorPaletteConfigSlice,
  querybuilderSlice,
  setQuerybuilderNodes,
} from '@graphpolaris/shared/data-access/store';
import { GraphPolarisThemeProvider } from '@graphpolaris/shared/data-access/theme';
import { AnyAction, configureStore, Store } from '@reduxjs/toolkit';
import {
  ComponentMeta,
  ComponentStory,
  ReactFramework,
} from '@storybook/react';
import { Provider } from 'react-redux';
import QueryBuilder from './querybuilder';
import { MultiGraph } from 'graphology';

export default {
  component: QueryBuilder,
} as ComponentMeta<typeof QueryBuilder>;

const Template: ComponentStory<typeof QueryBuilder> = (args) => (
  <QueryBuilder {...args} />
);

// Mock palette store
const mockStore = configureStore({
  reducer: {
    colorPaletteConfig: colorPaletteConfigSlice.reducer,
    querybuilder: querybuilderSlice.reducer,
  },
});
const graph = new MultiGraph();
graph.addNode('0', { type: 'entity', x: 100, y: 100, name: 'Entity Pill' });
graph.addNode('1', { type: 'relation', x: 140, y: 140, name: 'Relation Pill' });
graph.addNode('2', {
  type: 'attribute',
  x: 170,
  y: 160,
  name: 'Attr string',
  datatype: 'string',
  operator: 'EQ',
});
graph.addNode('3', {
  type: 'attribute',
  x: 170,
  y: 170,
  name: 'Attr number',
  datatype: 'float',
  operator: 'EQ',
});
graph.addNode('4', {
  type: 'attribute',
  x: 170,
  y: 180,
  name: 'Attr bool',
  datatype: 'bool',
  operator: 'EQ',
});
graph.addEdge('2', '1', { type: 'attribute_connection' });
graph.addEdge('3', '1', { type: 'attribute_connection' });
graph.addEdge('4', '1', { type: 'attribute_connection' });
// graph.addEdge('0', '1', { type: 'entity_relation' });
mockStore.dispatch(setQuerybuilderNodes(graph.export()));

export const NoData = Template.bind({});
NoData.decorators = [
  (story) => (
    <Provider store={mockStore}>
      <GraphPolarisThemeProvider>{story()}</GraphPolarisThemeProvider>
    </Provider>
  ),
];
