import { createReactFlowElements } from '@graphpolaris/querybuilder/usecases';
import {
  setQuerybuilderNodes,
  useAppDispatch,
  useQuerybuilderNodes,
} from '@graphpolaris/shared/data-access/store';
import {
  AllLayoutAlgorithms,
  LayoutFactory,
} from '@graphpolaris/shared/graph-layout';
import Graph, { MultiGraph } from 'graphology';
import { useEffect, useMemo, useState } from 'react';
import ReactFlow, {
  ReactFlowProvider,
  FlowElement,
  Background,
  Node,
} from 'react-flow-renderer';
import AttributeRFPill from './customFlowPills/attributepill/attributepill';
import EntityRFPill from './customFlowPills/entitypill/entitypill';
import RelationRFPill from './customFlowPills/relationpill/relationpill';

const nodeTypes = {
  entity: EntityRFPill,
  relation: RelationRFPill,
  attribute: AttributeRFPill,
};

const onLoad = (reactFlowInstance: any) => {
  setTimeout(() => reactFlowInstance.fitView(), 0);
};

const QueryBuilder = (props: {}) => {
  const nodes = useQuerybuilderNodes();
  const dispatch = useAppDispatch();

  const elements = useMemo(() => createReactFlowElements(nodes), [nodes]);

  const graphStyles = { width: '100%', height: '500px' };

  const onNodeDrag = (
    event: React.MouseEvent<Element, MouseEvent>,
    node: Node<any>
  ) => {
    const attr = nodes.getNodeAttributes(node.id);
    const dx = node.position.x - attr.x;
    const dy = node.position.y - attr.y;

    nodes.forEachInNeighbor(node.id, (nb) => {
      if (nodes.getNodeAttribute(nb, 'type') == 'attribute') {
        nodes.updateNodeAttribute(nb, 'x', (x) => x + dx);
        nodes.updateNodeAttribute(nb, 'y', (y) => y + dy);
      }
    });

    nodes.setNodeAttribute(node.id, 'x', node.position.x);
    nodes.setNodeAttribute(node.id, 'y', node.position.y);

    dispatch(setQuerybuilderNodes(nodes.export()));
  };

  return (
    <div>
      <ReactFlowProvider>
        <ReactFlow
          elements={elements}
          style={graphStyles}
          snapGrid={[10, 10]}
          // snapToGrid={true}
          nodeTypes={nodeTypes}
          onLoad={onLoad}
          onNodeDrag={onNodeDrag}
        >
          <Background gap={10} size={0.7} />
        </ReactFlow>
      </ReactFlowProvider>
    </div>
  );
};

export default QueryBuilder;
