import Graph from 'graphology';
import { Attributes } from 'graphology-types';
import { Elements, Node, Edge } from 'react-flow-renderer';

// Takes the querybuilder graph as an input and creates react flow elements for them.
export function createReactFlowElements(graph: Graph): Elements<Node | Edge> {
  const elements: Elements<Node | Edge> = [];

  graph.forEachNode((node: string, attributes: Attributes): void => {
    let data;

    switch (attributes.type) {
      case 'entity':
      case 'relation':
        data = { name: attributes.name };
        break;
      case 'attribute':
        data = {
          name: attributes.name,
          datatype: attributes.datatype,
          operator: attributes.operator,
        };
    }

    const RFNode: Node = {
      id: node,
      type: attributes.type,
      position: { x: attributes?.x || 0, y: attributes?.y || 0 },
      data: data,
    };
    elements.push(RFNode);
  });

  graph.forEachEdge((edge, attributes, source, target): void => {
    // connection from attributes don't have visible connection lines
    if (attributes.type == 'attribute_connection') return;

    const RFEdge: Edge = {
      id: edge,
      source: source,
      target: target,
      sourceHandle: attributes.sourceHandle,
      targetHandle: attributes.targetHandle,
    };
    elements.push(RFEdge);
  });

  return elements;
}
